/*
  Copyright 2018-2021, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2021, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  This file is part of GekkoFS.

  GekkoFS is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  GekkoFS is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GekkoFS.  If not, see <https://www.gnu.org/licenses/>.

  SPDX-License-Identifier: GPL-3.0-or-later
*/

#ifndef GEKKOFS_INFO_HPP
#define GEKKOFS_INFO_HPP

#include <memory>

namespace gkfs::api::fs {

// TODO: identical to preload_context.hpp FsConfig. Refactor?
struct config {
    // configurable metadata
    bool atime_state_;
    bool mtime_state_;
    bool ctime_state_;
    bool link_cnt_state_;
    bool blocks_state_;

    uid_t uid_;
    gid_t gid_;

    std::string rootdir_;
    std::string mountdir_;
};

class host {
public:
    host();
    ~host();
    host(const std::string& hostname, const std::string& uri);
    host(host&& rhs) noexcept;
    host(const host& other);

    host&
    operator=(host&& rhs) noexcept;
    host&
    operator=(const host& other);

    void
    swap(host& other);

    std::string
    hostname() const;
    std::string
    uri() const;

private:
    class impl;
    std::unique_ptr<impl> pimpl_;
};

} // namespace gkfs::api::fs

namespace gkfs::api::rpc {

struct config {
    std::string protocol_string_;
    bool auto_sm_enabled_{false};
};

} // namespace gkfs::api::rpc

#endif // GEKKOFS_INFO_HPP
