/*
  Copyright 2018-2021, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2021, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  This file is part of GekkoFS' POSIX interface.

  GekkoFS' POSIX interface is free software: you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  GekkoFS' POSIX interface is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with GekkoFS' POSIX interface.  If not, see
  <https://www.gnu.org/licenses/>.

  SPDX-License-Identifier: LGPL-3.0-or-later
*/

#include <memory>
#include <string>
#include <info.hpp>
#include <hermes/endpoint.hpp>
#include <utility>

namespace gkfs::api::fs {

class host::impl {
    friend class host;

public:
    impl() = default;
    impl(std::string hostname, std::string uri)
        : hostname_(std::move(hostname)), uri_(std::move(uri)) {}

private:
    std::string hostname_;
    std::string uri_;
    hermes::endpoint endpoint_;
};

host::host() : pimpl_(new host::impl()) {}
host::host(const host& other) : pimpl_(new host::impl(*other.pimpl_)) {}

host::host(const std::string& hostname, const std::string& uri)
    : pimpl_(new host::impl(hostname, uri)) {}

host::~host() = default;
host::host(host&&) noexcept = default;

host&
host::operator=(host&& rhs) noexcept = default;

host&
host::operator=(const host& other) {
    host tmp(other);
    this->swap(tmp);
    return *this;
}

void
host::swap(host& other) {
    pimpl_.swap(other.pimpl_);
}

std::string
host::hostname() const {
    return pimpl_->hostname_;
}

std::string
host::uri() const {
    return pimpl_->uri_;
}

} // namespace gkfs::api::fs

namespace std {

using gkfs::api::fs::host;

template <>
void
swap<host>(host& a, host& b) noexcept {
    a.swap(b);
}

} // namespace std