/*
  Copyright 2018-2021, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2021, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  This file is part of GekkoFS' POSIX interface.

  GekkoFS' POSIX interface is free software: you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation, either version 3 of the License,
  or (at your option) any later version.

  GekkoFS' POSIX interface is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with GekkoFS' POSIX interface.  If not, see
  <https://www.gnu.org/licenses/>.

  SPDX-License-Identifier: LGPL-3.0-or-later
*/

#include <vector>
#include <string>
#include <gekkofs.hpp>

#include <api/detail/connection_impl.hpp>

#include <fmt/format.h>

#include <global/path_util.hpp>
#include <global/metadata.hpp>

#include <client/rpc/forward_metadata.hpp> // TODO: api/rpc?

namespace gkfs::api::staging {

/**
 * Transfer data into a GekkoFS file.
 *
 * This function transfers data from the provided buffer into a target
 * file in GekkoFS in one operation. If the file does not exist, it will be
 * created. This function creates an internal connection to the filesystem and
 * destroys it after the operation completes.
 *
 * @param buffer The buffer with the data contents
 * @param size The amount of data to transfer
 * @param gkfs_pathname The target filename inside GekkoFS
 */
void
put(const void* buffer, size_t size,
    const std::filesystem::path& gkfs_filename, off_t offset = 0) {

    fs::connection fc;


    fc.put(buffer, size, gkfs_filename);
}

/**
 * Transfer data into a GekkoFS file.
 *
 * This function transfers data from the provided buffer into a target
 * file in GekkoFS in one operation. If the file does not exist, it will be
 * created. The function is capable of reusing an existing connection
 * to the filesystem, which might be beneficial for performance reasons, but
 * leaves the responsibility of creating and destroying the connection to the
 * user.
 *
 * @param conn An existing connection for GekkoFS
 * @param buffer The buffer with the data contents
 * @param size The amount of data to transfer
 * @param gkfs_pathname The target filename inside GekkoFS
 */
void
put(const gkfs::api::fs::connection& conn, const void* buffer, size_t size,
    const std::filesystem::path& gkfs_filename) {}

void
put(const std::filesystem::path& pathname, const void* buf, size_t count,
    off_t offset) {}

void
put(const gkfs::api::fs::connection& conn,
    const std::filesystem::path& pathname, const void* buf, size_t count,
    off_t offset) {}

} // namespace gkfs::api::staging